@@ -10,21 +10,128 @@ from warnings import warn
 from time import sleep
 from collections import defaultdict
 from shapely.geometry import shape
-from .helpers import is_number
+from .helpers import is_number, get_buffer_size
 from threading import Thread
+from shapely.ops import unary_union
+from shapely.geometry import shape
 
 class GEOGRIDDATA(list):
+	'''
+	Class to package the input needed by each indicator. 
+	This class extends a simple list to charge it with additional properties, if needed.
+	It's mainly used for internal purposes. 
+
+	Parameters
+	----------
+	geogrid_data : list
+		List to converg to GEOGRIDDATA object.
+	'''
 	def __init__(self,geogrid_data):
 		super(GEOGRIDDATA, self).__init__()
+		if isinstance(geogrid_data,dict):
+			raise NameError('Invalid GEOGRIDDATA endpoint. You need to update your grid at least once. See Handler.reset_geogrid_data()')
 		for e in geogrid_data:
 			self.append(e)
 		self.geogrid_props = None
+		self.GEOGRID = None
 
-	def set_geogrid_props(self,geogrid_props):
-		self.geogrid_props = geogrid_props
+	def link_table(self,table_name):
+		'''
+		Sets geogrid using set_geogrid.
+		This function should use if GEOGRIDDATA needs to be updated.
+
+		Parameters
+		----------
+		table_name: str or :class:`brix.Handler`
+			Name of the table or Handler object.
+		'''
+		if isinstance(table_name,Handler):
+			tableHandler = table_name
+		else:
+			tableHandler = Handler(table_name)
+		self.set_geogrid(tableHandler.get_GEOGRID())
+
+	def set_geogrid(self,GEOGRID):
+		self.GEOGRID = GEOGRID
+
+	def get_geogrid(self):
+		'''
+		Get the value of GEOGRIDDATA from the corresponding :class:`brix.Handler`.
+
+		Returns
+		-------
+		GEOGRID : dict
+			Value of GEOGRID
+		'''
+		return self.GEOGRID
 
 	def get_geogrid_props(self):
-		return self.geogrid_props
+		'''
+		Get the value of :attr:`brix.Handler.geogrid_props` from the corresponding :class:`brix.Handler`.
+
+		Returns
+		-------
+		geogrid_props : dict or list
+			Value of :attr:`brix.Handler.geogrid_props`
+		'''
+		return self.GEOGRID['properties']
+
+	def grid_size(self):
+		return len(self.get_geogrid()['features'])
+
+	def get_type_info(self):
+		return self.get_geogrid_props()['types']
+
+	def get_type_set(self):
+		return set(self.get_geogrid_props()['types'])
+
+	def number_of_types(self):
+		return len(self.get_type_set())
+
+	def check_type_validity(self,quietly=True):
+		non_defined_cells = set([cell['name'] for cell in self]).difference(self.get_type_set())
+		if len(non_defined_cells)==0:
+			return True
+		else:
+			if not quietly:
+				print('Unrecognized types:',non_defined_cells)
+			return False
+
+	def check_id_validity(self,quietly=True):
+		n_unique_ids = len(set([cell['id'] for cell in self]))
+		if n_unique_ids==self.grid_size():
+			return True
+		else:
+			if not quietly:
+				print('Number of unique cells in geogrid_data does not match grid size')
+			return False
+
+	def fill_missing_cells(self):
+		'''
+		Fills missing cells from GEOGRID.
+
+		This is useful when working only with interactive cells.
+		'''
+		available_ids = [cell['id'] for cell in self]
+		for cell in self.GEOGRID['features']:
+			cell = cell['properties']
+			if cell['id'] not in available_ids:
+				self.append(cell)
+
+	def remap_colors(self):
+		'''
+		Forces the colors to match the define colors of the cell type. 
+		Requires that GEOGRIDDATA is set
+		'''
+		if self.GEOGRID is None:
+			raise NameError('GEOGRIDDATA object does not have GEOGRID attribute.')
+		if not self.check_type_validity(quietly=False):
+			raise NameError('Type not found in table definition.')
+		GEOGRID = self.GEOGRID
+		for cell in self:
+			h = GEOGRID['properties']['types'][cell['name']]['color'].replace('#','')
+			color = list(int(h[i:i+2], 16) for i in (0, 2, 4))
+			cell['color'] = color
 
 class Handler(Thread):
 	'''Class to handle the connection for indicators built based on data from the GEOGRID. To use, instantiate the class and use the :func:`~brix.Handler.add_indicator` method to pass it a set of :class:`~brix.Indicator` objects.
@@ -75,6 +182,7 @@ class Handler(Thread):
 		self.GEOGRID = None
 
 		self.indicators = {}
+		self.update_geogrid_data_functions = []
 		self.grid_hash_id = None
 		self.grid_hash_id = self.get_grid_hash()
 
@@ -84,7 +192,47 @@ class Handler(Thread):
 		self.none_character = 0
 		self.geogrid_props=None
 
-		self.reference =reference
+		self.reference = reference
+		self.classification_list = ['LBCS','NAICS']
+
+		self.OSM_data = {}
+
+	def grid_bounds(self,bbox=False,buffer_percent=None):
+		'''
+		Returns the bounds of the geogrid.
+
+		Parameters
+		----------
+		bbox: boolean, defaults to False
+			If True, it will return a bounding box instead of a polygon. [W, S, E, N]
+		buffer_percent: float, optional
+			If given, this will add a buffer around the table.
+			Size of buffer in units of the grid diameter
+			See :func:`brix.get_buffer_size`.
+
+		Returns
+		-------
+		limit: shapely.Polygon or list
+			Bounds of the table. If `bbox=True` it will return a horizontal bounding box.
+		'''
+		geogrid_data = self._get_grid_data(include_geometries=True)
+
+		grid = [shape(cell['geometry']) for cell in geogrid_data]
+		limit = unary_union(grid)
+		limit = limit.buffer(get_buffer_size(limit,buffer_percent=0.001))
+		limit = limit.simplify(0.00001)
+
+		if buffer_percent is not None:
+			buffer_size = get_buffer_size(limit,buffer_percent=buffer_percent)
+			limit = limit.buffer(buffer_size)
+			limit = limit.simplify(0.0001)
+
+		if bbox:
+			lons,lats = zip(*limit.exterior.coords)
+			return [min(lons),min(lats),max(lons),max(lats)]
+		else:
+			return limit
+
         
 	def check_table(self,return_value=False):
 		'''Prints the front end url for the table. 
@@ -117,10 +265,12 @@ class Handler(Thread):
 			Current value of selected indicators.
 		'''
 		if indicator_type in ['numeric']:
-			print(self.cityIO_get_url+'/indicators')
+			if not self.quietly:
+				print(self.cityIO_get_url+'/indicators')
 			r = self._get_url(self.cityIO_get_url+'/indicators')
 		elif indicator_type in ['heatmap','access']:
-			print(self.cityIO_get_url+'/access')
+			if not self.quietly:
+				print(self.cityIO_get_url+'/access')
 			r = self._get_url(self.cityIO_get_url+'/access')
 		else:
 			raise NameError('Indicator type should either be numeric, heatmap, or access. Current type: '+str(indicator_type))
@@ -196,15 +346,31 @@ class Handler(Thread):
 				raise NameError('Indicator type should either be numeric, heatmap, or access. Current type: '+str(I.indicator_type))
 			try:
 				if I.is_composite:
-					indicator_values = self.get_indicator_values(include_composite=False)
+					indicator_values = self.get_indicator_values(geogrid_data=geogrid_data,include_composite=False)
 					self._new_value(indicator_values,indicatorName)
 				else:
 					self._new_value(geogrid_data,indicatorName)
 			except:
 				warn('Indicator not working: '+indicatorName)
 
+	def add_geogrid_data_update_function(self,update_func):
+		'''
+		Adds a function to update GEOGRIDDATA. 
+
+		See :func:`brix.Handler.update_geogrid_data`.
+
+		Parameters
+		----------
+		update_func : function
+			Function to update the geogriddadata (list of dicts)
+			Function should take a :class:`brix.Handler` as the first and only positional argument.
+			No keyword arguments are supported when using this feature.
+			Function should return a list of dicts that represents a valid geogriddata object.
+		'''
+		self.update_geogrid_data_functions.append(update_func)
+
 	def return_indicator(self,indicator_name):
-		'''Returns the value returned by :func:`brix.Indicator.return_indicator` function of the selected indicator.
+		'''Returns the unformatted value returned by :func:`brix.Indicator.return_indicator` function of the selected indicator.
 
 		Parameters
 		----------
@@ -219,7 +385,7 @@ class Handler(Thread):
 		geogrid_data = self._get_grid_data()
 		I = self.indicators[indicator_name]
 		if I.is_composite:
-			indicator_values = self.get_indicator_values(include_composite=False)
+			indicator_values = self.get_indicator_values(geogrid_data=geogrid_data,include_composite=False)
 			return I.return_indicator(indicator_values)
 		else:
 			return I.return_indicator(geogrid_data)
@@ -346,7 +512,6 @@ class Handler(Thread):
 					new_value['indicator_type'] = I.indicator_type
 				if ('viz_type' not in new_value.keys())&(I.viz_type is not None):
 					new_value['viz_type'] = I.viz_type
-				print(new_value)
 				return [new_value]
 
 
@@ -378,7 +543,7 @@ class Handler(Thread):
 
 		return {'type':'FeatureCollection','properties':all_properties,'features':combined_features}
 
-	def get_indicator_values(self,include_composite=False):
+	def get_indicator_values(self,geogrid_data=None,include_composite=False):
 		'''
 		Returns the current values of numeric indicators. Used for developing a composite indicator.
 
@@ -392,7 +557,8 @@ class Handler(Thread):
 		indicator_values : dict
 			Dictionary with values for each indicator formatted as: ``{indicator_name: indicator_value, ...}``
 		'''
-		geogrid_data = self.get_geogrid_data()
+		if geogrid_data is None:
+			geogrid_data = self._get_grid_data()
 		new_values_numeric = []
 		for indicator_name in self.indicators:
 			I = self.indicators[indicator_name]
@@ -471,7 +637,7 @@ class Handler(Thread):
 		geogrid_data = self._get_grid_data()
 		for indicator_name in self.indicators:
 			if self.indicators[indicator_name].is_composite:
-				indicator_values = self.get_indicator_values(include_composite=False)
+				indicator_values = self.get_indicator_values(geogrid_data=geogrid_data,include_composite=False)
 				self._new_value(indicator_values,indicator_name)
 			else:
 				self._new_value(geogrid_data,indicator_name)
@@ -486,12 +652,8 @@ class Handler(Thread):
 			Table GEOGRID properties.
 		'''
 		if self.geogrid_props is None:
-			r = self._get_url(self.cityIO_get_url+'/GEOGRID/properties')
-			if r.status_code==200:
-				self.geogrid_props = r.json()
-			else:
-				warn('Cant access cityIO type definitions')
-				sleep(1)
+			geogrid = self.get_GEOGRID()
+			self.geogrid_props = geogrid['properties']
 		return self.geogrid_props
 
 	def get_table_properties(self):
@@ -519,36 +681,104 @@ class Handler(Thread):
 			grid_hash_id=self.grid_hash_id
 		return grid_hash_id
 
-	def _get_grid_data(self,include_geometries=False,with_properties=False):
+	def get_GEOGRID(self):
+		if self.GEOGRID is None:
+			r = self._get_url(self.cityIO_get_url+'/'+self.GEOGRID_varname)
+			if r.status_code==200:
+				geogrid = r.json()
+				try:
+					geogrid = self.parse_classifications(geogrid)
+				except:
+					warn('NAICS and LBCS classifications were not properly parsed.')
+				self.GEOGRID = geogrid
+			else:
+				warn('WARNING: Cant access GEOGRIDDATA')
+		return self.GEOGRID
+
+	def normalize_codes(self,code_proportion):
+		'''
+		Helper function to transform:
+		[{'proportion': 0.3, 'use': {'6700': 1}}, {'proportion': 0.7, 'use': {'2310': 0.3, '4100': 0.7}}]
+
+		into:
+		{'6700': 0.3, '2310': 0.21, '4100': 0.49}
+		'''
+		new_code_proportion = defaultdict(lambda: 0)
+		for prop in code_proportion:
+			for code in prop['use']:
+				new_code_proportion[code]+= round(prop['proportion']*prop['use'][code],5)
+		new_code_proportion = dict(new_code_proportion)
+		total = sum(new_code_proportion.values())
+		new_code_proportion = {k:new_code_proportion[k]/total for k in new_code_proportion}
+		return new_code_proportion
+
+	def parse_classifications(self,geogrid):
+		'''
+		Helper function to parse the LBCS and NAICS strings into dictionaries of the form:
+		{'6700': 0.3, '2310': 0.21, '4100': 0.49}
+		'''
+		for t in geogrid['properties']['types']:
+			for code in self.classification_list:
+				if code in geogrid['properties']['types'][t].keys():
+					code_proportion = geogrid['properties']['types'][t][code]
+				else:
+					code_proportion = None
+				if (code_proportion is not None) and (code_proportion !='null'):
+					if isinstance(geogrid['properties']['types'][t][code],str):
+						code_proportion = json.loads(geogrid['properties']['types'][t][code])
+					code_proportion = self.normalize_codes(code_proportion)
+				else:
+					code_proportion = None
+				geogrid['properties']['types'][t][code] = code_proportion
+		return geogrid
+
+	def get_GEOGRIDDATA(self):
+		'''
+		Returns the raw GEOGRIDDATA object.
+		This function should be treated as a low-level function, please use :func:`brix.Handler.get_geogrid_data` instead.
+		'''
 		r = self._get_url(self.cityIO_get_url+'/'+self.GEOGRIDDATA_varname)
 		if r.status_code==200:
 			geogrid_data = r.json()
 		else:
-			warn('WARNING: Cant access GEOGRID data')
+			warn('WARNING: Cant access GEOGRIDDATA')
 			sleep(1)
 			geogrid_data = None
+		return geogrid_data
+
+	def _get_grid_data(self,include_geometries=False,with_properties=False,exclude_noninteractive=False):
+		geogrid_data = self.get_GEOGRIDDATA()
+		geogrid = self.get_GEOGRID()
+		
+		geogrid_data = GEOGRIDDATA(geogrid_data)
+		geogrid_data.set_geogrid(geogrid)
+		if not geogrid_data.check_id_validity():
+			geogrid_data.fill_missing_cells()
+			if not geogrid_data.check_id_validity():
+				warn('WARNING: Current GEOGRIDDATA includes undefined types.')
 	
 		if include_geometries|any([I.requires_geometry for I in self.indicators.values()]):
-			if self.GEOGRID is None:
-				r = self._get_url(self.cityIO_get_url+'/'+self.GEOGRID_varname)
-				if r.status_code==200:
-					geogrid = r.json()
-					self.GEOGRID = geogrid
-				else:
-					warn('WARNING: Cant access GEOGRID data')
 			for i in range(len(geogrid_data)):
-				geogrid_data[i]['geometry'] = self.GEOGRID['features'][i]['geometry']
+				geogrid_data[i]['geometry'] = self.get_GEOGRID()['features'][i]['geometry']
 
 		if with_properties|any([I.requires_geogrid_props for I in self.indicators.values()]):
-			geogrid_props = self.get_geogrid_props()
+			geogrid_props = geogrid['properties']
 			types_def = geogrid_props['types'].copy()
 			if 'static_types' in geogrid_props:
 				types_def.update(geogrid_props['static_types'])
 			types_def['None'] = None
 			for cell in geogrid_data:
-				cell['properties'] = types_def[cell['name']]
-		geogrid_data = GEOGRIDDATA(geogrid_data)
-		geogrid_data.set_geogrid_props(self.get_geogrid_props())
+				if cell['name'] in types_def.keys():
+					cell['properties'] = types_def[cell['name']]
+
+		if exclude_noninteractive:
+			interactive_geogrid_data = []
+			for cell in geogrid_data:
+				if 'interactive' in cell.keys():
+					if cell['interactive']=='Web':
+						interactive_geogrid_data.append(cell)
+			geogrid_data = interactive_geogrid_data
+
 		return geogrid_data
 
 	def _get_url(self,url,params=None):
@@ -566,7 +796,7 @@ class Handler(Thread):
 			warn('FAILED TO RETRIEVE URL: '+url)
 		return r
 
-	def get_geogrid_data(self,include_geometries=False,with_properties=False,as_df=False):
+	def get_geogrid_data(self,include_geometries=False,with_properties=False,exclude_noninteractive=False,as_df=False):
 		'''
 		Returns the geogrid data from:
 		http://cityio.media.mit.edu/api/table/table_name/GEOGRIDDATA
@@ -577,6 +807,8 @@ class Handler(Thread):
 			If `True` it will also add the geometry information for each grid unit.
 		with_properties : boolean, defaults to `False`
 			If `True` it will add the properties of each grid unit as defined when the table was constructed (e.g. LBCS code, NAICS code, etc.)
+		exclude_noninteractive : boolean, defaults to `False`
+			If `True` it will exclude non-interactive cells. 
 		as_df: boolean, defaults to `False`
 			If `True` it will return data as a pandas.DataFrame.
 
@@ -585,20 +817,30 @@ class Handler(Thread):
 		geogrid_data : dict
 			Data taken directly from the table to be used as input for :class:`brix.Indicator.return_indicator`.
 		'''
-		geogrid_data = self._get_grid_data(include_geometries=include_geometries,with_properties=with_properties)
+		geogrid_data = self._get_grid_data(include_geometries=include_geometries,with_properties=with_properties,exclude_noninteractive=exclude_noninteractive)
+
 		if as_df:
 			for cell in geogrid_data:
 				if 'properties' in cell.keys():
 					cell_props = cell['properties']
 					for k in cell_props:
-						cell[f'property_{k}'] = cell_props[k]
+						if cell_props[k] is not None:
+							if k not in self.classification_list:
+								cell[f'property_{k}'] = cell_props[k]
+							else:
+								for code in cell_props[k]:
+									cell[f'{k}_{code}'] = cell_props[k][code]
 					del cell['properties']
 			geogrid_data = pd.DataFrame(geogrid_data)
+			columns_order = [c for c in geogrid_data.columns if c.split('_')[0] not in self.classification_list]
+			columns_order+= sorted([c for c in geogrid_data.columns if c.split('_')[0] in self.classification_list])
+			geogrid_data = geogrid_data[columns_order]
 			if include_geometries:
 				geogrid_data = gpd.GeoDataFrame(geogrid_data.drop('geometry',1),geometry=geogrid_data['geometry'].apply(lambda x: shape(x)))
+
 		return geogrid_data
 
-	def perform_update(self,grid_hash_id=None,append=True):
+	def perform_update(self,grid_hash_id=None,append=False):
 		'''
 		Performs single table update.
 
@@ -611,13 +853,13 @@ class Handler(Thread):
 		'''
 		if grid_hash_id is None: 
 			grid_hash_id = self.get_grid_hash()	
-		geogrid_data = self._get_grid_data()
 		if not self.quietly:
 			print('Updating table with hash:',grid_hash_id)
 
-		new_values = self.update_package(geogrid_data=geogrid_data,append=append)
+		new_values = self.update_package(append=append)
 
 		if len(new_values['numeric'])!=0:
+
 			r = requests.post(self.cityIO_post_url+'/indicators', data = json.dumps(new_values['numeric']))
 
 		if len(new_values['heatmap']['features'])!=0:
@@ -626,6 +868,22 @@ class Handler(Thread):
 			print('Done with update')
 		self.grid_hash_id = grid_hash_id
 
+	def perform_geogrid_data_update(self):
+		'''
+		Performs GEOGRIDDATA update using the functions added to the :class:`brix.Handler` using :func:`brix.Hanlder.add_geogrid_data_update_function`.
+
+		Returns True if an update happened, and Flase otherwise.
+		'''
+		update_flag = False
+		for update_func in self.update_geogrid_data_functions:
+			new_geogrid_data = update_func(self)
+			self.post_geogrid_data(new_geogrid_data)
+			update_flag = True
+			if not self.quietly:
+				print('GEOGRIDDATA successfully updated')
+		return update_flag
+
+
 	def rollback(self):
 		''':class:`brix.Handler` keeps track of the previous value of the indicators and access values.This function rollsback the current values to whatever the locally stored values are.
 		See also :func:`brix.Handler.previous_indicators` and :func:`brix.Handler.previous_access`.
@@ -664,16 +922,19 @@ class Handler(Thread):
 
 		if not self.quietly:
 			print('Performing initial update')
-			print('Update package example:')
-			print(self.update_package())
+
+		self.perform_geogrid_data_update()
 		self.perform_update(append=self.append_on_post)
 
 		if showFront:
 			webbrowser.open(self.front_end_url, new=2)
+		self.grid_hash_id = self.get_grid_hash()
 		while True:
 			sleep(self.sleep_time)
 			grid_hash_id = self.get_grid_hash()
 			if grid_hash_id!=self.grid_hash_id:
+				if self.perform_geogrid_data_update():
+					grid_hash_id = self.get_grid_hash()
 				self.perform_update(grid_hash_id=grid_hash_id,append=self.append_on_post)
 
 	def run(self):
@@ -688,7 +949,8 @@ class Handler(Thread):
 		Listens for changes in the table's geogrid and update all indicators accordingly. 
 		You can use the update_package method to see the object that will be posted to the table.
 		This method starts with an update before listening.
-		This runs in a separate thread by default.
+		Can run in a separate thread.
+		Does not support updating GEOGRIDDATA.
 
 		Parameters
 		----------
@@ -708,6 +970,84 @@ class Handler(Thread):
 		else:
 			self._listen(showFront=showFront)
 
+	def reset_geogrid_data(self):
+		'''
+		Resets the GEOGRIDDATA endpoint to the initial value.
+		If the GEOGRIDDATA has not been updated, this will update it. 
+		'''
+		geogrid_data = []
+		for i,cell in enumerate(self.get_GEOGRID()['features']):
+			cell = cell['properties']
+			cell['id'] = i
+			geogrid_data.append(cell)
+		self.post_geogrid_data(geogrid_data)
+
+	def post_geogrid_data(self,geogrid_data):
+		'''
+		Posts the given geogrid_data object, ensuring that the object is valid.
+
+		Function can be called by itself or using :func:`brix.Handler.update_geogrid_data`.
+
+		Parameters
+		----------
+		geogrid_data: dict
+			Dictionary corresponding to a valid :class:`brix.GEOGRIDDATA` object.
+		'''
+		geogrid_data = GEOGRIDDATA(geogrid_data)
+		geogrid_data.set_geogrid(self.get_GEOGRID())
+
+		if not geogrid_data.check_type_validity(quietly=False):
+			raise NameError('Type not found in table definition.')
+
+		if not geogrid_data.check_id_validity():
+			geogrid_data.fill_missing_cells()
+			if not geogrid_data.check_id_validity():
+				raise NameError('IDs do not match.')
+
+		geogrid_data.remap_colors()
+
+		geogrid_data = list(geogrid_data)
+
+		ids = [cell['id'] for cell in geogrid_data]
+		ids = np.argsort(ids)
+		geogrid_data = [geogrid_data[i] for i in ids]
+
+		r = requests.post(self.cityIO_post_url+'/'+self.GEOGRIDDATA_varname, data=json.dumps(geogrid_data))
+		self.grid_hash_id = self.get_grid_hash()
+		if not self.quietly:
+			print('GEOGRIDDATA successfully updated:',self.grid_hash_id)
+
+
+	def update_geogrid_data(self, update_func, **kwargs):
+		'''
+		Function to update table GEOGRIDDATA.
+
+		Parameters
+		----------
+		update_func : function
+			Function to update the geogriddadata (list of dicts)
+			Function should take a :class:`brix.Handler` as the first and only positional argument plus any number of keyword arguments.
+			Function should return a list of dicts that represents a valid geogriddata object.
+
+		Example
+		-------
+		>>> def add_height(H, levels=1):
+				geogrid_data = H.get_geogrid_data()
+				for cell in geogrid_data:
+					cell['height'] += levels
+				return geogrid_data
+		>>> levels = 3
+		>>> H = Handler('tablename', quietly=False)
+		>>> H.update_geogrid_data(add_height, levels=levels)
+		'''
+
+		new_geogrid_data = update_func(self, **kwargs)
+
+		self.post_geogrid_data(new_geogrid_data)
+		if not self.quietly:
+			print('Done with update')
+
+
 
 class Indicator:
 	'''Parent class to build indicators from. To use, you need to define a subclass than inherets properties from this class. Doing so, ensures your indicator inherets the necessary methods and properties to connect with a CityScipe table.'''
@@ -717,6 +1057,7 @@ class Indicator:
 		self.viz_type = 'radar'
 		self.requires_geometry = None
 		self.requires_geogrid_props = False
+		self.exclude_noninteractive = False
 		self.model_path = None
 		self.pickled_model = None
 		# self.int_types_def=None
@@ -738,6 +1079,8 @@ class Indicator:
 			else:
 				self.requires_geometry = False
 
+		self.return_indicator_user = None
+
 
 	def setup(self):
 		'''User defined function. Used to set up the main attributed of the custom indicator. Acts similar to an `__init__` method.'''
@@ -756,7 +1099,21 @@ class Indicator:
 		indicator_value : list, dict, or float
 			Value of indicator or list of values. When returning a dict, please use the format ``{'name': 'Indicator Name', 'value': indicator_value}``. When returning a list, please return a list of dictionaries in the same format. 
 		'''
-		return {}
+		if self.return_indicator_user is not None:
+			return self.return_indicator_user(geogrid_data)
+		else:
+			return {}
+
+	def set_return_indicator(self,return_indicator):
+		'''
+		Used to set the return_indicator method by passing a function.
+
+		Parameters
+		----------
+		return_indicator: func
+			Function that takes `geogrid_data` as input.
+		'''
+		self.return_indicator_user = return_indicator
 
 
 	def load_module(self):
@@ -809,7 +1166,7 @@ class Indicator:
 		return self.tableHandler.get_geogrid_props()['header']
 
 
-	def get_geogrid_data(self,as_df=False,include_geometries=None,with_properties=None):
+	def get_geogrid_data(self,as_df=False,include_geometries=None,with_properties=None,exclude_noninteractive=None):
 		'''
 		Returns the geogrid data from the linked table. Function mainly used for development. See :func:`brix.Indicator.link_table`. It returns the exact object that will be passed to return_indicator
 
@@ -822,14 +1179,18 @@ class Indicator:
 			If `True`, it will override the default parameter of the Indicator.
 		with_properties: boolean, defaults to :attr:`brix.Indicator.requires_geogrid_props`
 			If `True`, it will override the default parameter of the Indicator.
+		exclude_noninteractive : boolean, defaults to `False`
+			If `True` it will exclude non-interactive cells. 
 
 		Returns
 		-------
 		geogrid_data : str or pandas.DataFrame
 			Data that will be passed to the :func:`brix.Indicator.return_indicator` function by the :class:`brix.Handler` when deployed.
 		'''
-		include_geometries = self.requires_geometry if include_geometries is None else include_geometries
-		with_properties    = self.requires_geogrid_props if with_properties is None else with_properties
+		include_geometries     = self.requires_geometry if include_geometries is None else include_geometries
+		with_properties        = self.requires_geogrid_props if with_properties is None else with_properties
+		exclude_noninteractive = self.exclude_noninteractive if exclude_noninteractive is None else exclude_noninteractive
+		
 		if self.tableHandler is None:
 			if self.table_name is not None:
 				self.link_table(table_name=self.table_name)
@@ -837,7 +1198,7 @@ class Indicator:
 				warn('To use this function, please link a table first:\n> Indicator.link_table(table_name)')
 				return None
 
-		geogrid_data = self.tableHandler._get_grid_data(include_geometries=include_geometries,with_properties=with_properties)
+		geogrid_data = self.tableHandler._get_grid_data(include_geometries=include_geometries,with_properties=with_properties,exclude_noninteractive=exclude_noninteractive)
 		if as_df:
 			geogrid_data = pd.DataFrame(geogrid_data)
 			if include_geometries:
